@@ -3,14 +3,20 @@ package DBIx::Class::Schema::Loader;
 use strict;
 use warnings;
 use base qw/DBIx::Class::Schema Class::Accessor::Grouped/;
+use MRO::Compat;
+use mro 'c3';
 use Carp::Clan qw/^DBIx::Class/;
-use Class::C3;
-use Scalar::Util qw/ weaken /;
+use Scalar::Util 'weaken';
+use Sub::Name 'subname';
+use DBIx::Class::Schema::Loader::Utils 'array_eq';
+use Try::Tiny;
+use Hash::Merge 'merge';
+use namespace::clean;
 
 # Always remember to do all digits for the version even if they're 0
 # i.e. first release of 0.XX *must* be 0.XX000. This avoids fBSD ports
 # brain damage and presumably various other packaging systems too
-our $VERSION = '0.04999_14';
+our $VERSION = '0.07034_01';
 
 __PACKAGE__->mk_group_accessors('inherited', qw/
                                 _loader_args
@@ -23,9 +29,11 @@ __PACKAGE__->mk_group_accessors('inherited', qw/
 /);
 __PACKAGE__->_loader_args({});
 
+=encoding UTF-8
+
 =head1 NAME
 
-DBIx::Class::Schema::Loader - Dynamic definition of a DBIx::Class::Schema
+DBIx::Class::Schema::Loader - Create a DBIx::Class::Schema based on a database
 
 =head1 SYNOPSIS
 
@@ -38,12 +46,15 @@ DBIx::Class::Schema::Loader - Dynamic definition of a DBIx::Class::Schema
       { debug => 1,
         dump_directory => './lib',
       },
-      [ 'dbi:Pg:dbname="foo"', 'myuser', 'mypassword' ],
+      [ 'dbi:Pg:dbname="foo"', 'myuser', 'mypassword',
+         { loader_class => 'MyLoader' } # optionally
+      ],
   );
 
   # from the command line or a shell script with dbicdump (distributed
   # with this module).  Do `perldoc dbicdump` for usage.
   dbicdump -o dump_directory=./lib \
+           -o components='["InflateColumn::DateTime"]' \
            -o debug=1 \
            My::Schema \
            'dbi:Pg:dbname=foo' \
@@ -70,37 +81,47 @@ DBIx::Class::Schema::Loader - Dynamic definition of a DBIx::Class::Schema
   # -or-
   my $schema1 = "My::Schema"; $schema1->connection(as above);
 
-=head1 DESCRIPTION 
+=head1 DESCRIPTION
 
 DBIx::Class::Schema::Loader automates the definition of a
-L<DBIx::Class::Schema> by scanning database table definitions and
-setting up the columns, primary keys, and relationships.
+L<DBIx::Class::Schema> by scanning database table definitions and setting up
+the columns, primary keys, unique constraints and relationships.
 
-DBIx::Class::Schema::Loader currently supports only the DBI storage type.
-It has explicit support for L<DBD::Pg>, L<DBD::mysql>, L<DBD::DB2>,
-L<DBD::SQLite>, and L<DBD::Oracle>.  Other DBI drivers may function to
-a greater or lesser degree with this loader, depending on how much of the
-DBI spec they implement, and how standard their implementation is.
+See L<dbicdump> for the C<dbicdump> utility.
+
+DBIx::Class::Schema::Loader currently supports only the DBI storage type. It
+has explicit support for L<DBD::Pg>, L<DBD::mysql>, L<DBD::DB2>,
+L<DBD::Firebird>, L<DBD::InterBase>, L<DBD::Informix>, L<DBD::SQLAnywhere>,
+L<DBD::SQLite>, L<DBD::Sybase> (for Sybase ASE and MSSSQL), L<DBD::ODBC> (for
+MSSQL, MSAccess, Firebird and SQL Anywhere) L<DBD::ADO> (for MSSQL and
+MSAccess) and L<DBD::Oracle>.  Other DBI drivers may function to a greater or
+lesser degree with this loader, depending on how much of the DBI spec they
+implement, and how standard their implementation is.
 
 Patches to make other DBDs work correctly welcome.
 
 See L<DBIx::Class::Schema::Loader::DBI::Writing> for notes on writing
 your own vendor-specific subclass for an unsupported DBD driver.
 
-This module requires L<DBIx::Class> 0.07006 or later, and obsoletes
-the older L<DBIx::Class::Loader>.
+This module requires L<DBIx::Class> 0.08127 or later, and obsoletes the older
+L<DBIx::Class::Loader>.
 
-This module is designed more to get you up and running quickly against
-an existing database, or to be effective for simple situations, rather
-than to be what you use in the long term for a complex database/project.
-
-That being said, transitioning your code from a Schema generated by this
-module to one that doesn't use this module should be straightforward and
-painless, so don't shy away from it just for fears of the transition down
-the road.
+See L<DBIx::Class::Schema::Loader::Base> for available options.
 
 =head1 METHODS
 
+=head2 loader
+
+The loader object, as class data on your Schema. For methods available see
+L<DBIx::Class::Schema::Loader::Base> and L<DBIx::Class::Schema::Loader::DBI>.
+
+=cut
+
+sub loader {
+    my $self = shift;
+    $self->_loader(@_);
+}
+
 =head2 loader_class
 
 =over 4
@@ -153,26 +174,97 @@ sub _invoke_loader {
 
     my $args = $self->_loader_args;
 
-    # set up the schema/schema_class arguments
-    $args->{schema} = $self;
+    # temporarily copy $self's storage to class
+    my $class_storage = $class->storage;
+    if (ref $self) {
+        $class->storage($self->storage);
+        $class->storage->set_schema($class);
+    }
+
+    $args->{schema} = $class;
     $args->{schema_class} = $class;
-    weaken($args->{schema}) if ref $self;
     $args->{dump_directory} ||= $self->dump_to_dir;
     $args->{naming} = $self->naming if $self->naming;
-    $args->{use_namespaces} = $self->use_namespaces if $self->use_namespaces;
+    $args->{use_namespaces} = $self->use_namespaces if defined $self->use_namespaces;
 
     # XXX this only works for relative storage_type, like ::DBI ...
-    my $impl = $self->loader_class
-      || "DBIx::Class::Schema::Loader" . $self->storage_type;
-    $impl = "DBIx::Class::Schema::Loader${impl}" if $impl =~ /^::/;
-    eval { $self->ensure_class_loaded($impl) };
-    croak qq/Could not load storage_type loader "$impl": "$@"/ if $@;
+    my $loader_class = $self->loader_class;
+    if ($loader_class) {
+        $loader_class = "DBIx::Class::Schema::Loader${loader_class}" if $loader_class =~ /^::/;
+        $args->{loader_class} = $loader_class;
+    };
 
-    $self->_loader($impl->new(%$args));
-    $self->_loader->load;
-    $self->_loader_invoked(1);
+    my $impl = $loader_class || "DBIx::Class::Schema::Loader" . $self->storage_type;
+    try {
+        $self->ensure_class_loaded($impl)
+    }
+    catch {
+        croak qq/Could not load loader_class "$impl": "$_"/;
+    };
 
-    $self;
+    $class->loader($impl->new(%$args));
+    $class->loader->load;
+    $class->_loader_invoked(1);
+
+    # copy to $self
+    if (ref $self) {
+        $self->loader($class->loader);
+        $self->_loader_invoked(1);
+
+        $self->_merge_state_from($class);
+    }
+
+    # restore $class's storage
+    $class->storage($class_storage);
+
+    return $self;
+}
+
+# FIXME This needs to be moved into DBIC at some point, otherwise we are
+# maintaining things to do with DBIC guts, which we have no business of
+# maintaining. But at the moment it would be just dead code in DBIC, so we'll
+# maintain it here.
+sub _merge_state_from {
+    my ($self, $from) = @_;
+
+    my $orig_class_mappings       = $self->class_mappings;
+    my $orig_source_registrations = $self->source_registrations;
+
+    $self->_copy_state_from($from);
+
+    $self->class_mappings(merge($orig_class_mappings, $self->class_mappings))
+        if $orig_class_mappings;
+
+    $self->source_registrations(merge($orig_source_registrations, $self->source_registrations))
+        if $orig_source_registrations;
+}
+
+sub _copy_state_from {
+    my $self = shift;
+    my ($from) = @_;
+
+    # older DBIC's do not have this method
+    if (try { DBIx::Class->VERSION('0.08197'); 1 }) {
+        return $self->next::method(@_);
+    }
+    else {
+        # this is a copy from DBIC git master pre 0.08197
+        $self->class_mappings({ %{$from->class_mappings} });
+        $self->source_registrations({ %{$from->source_registrations} });
+
+        foreach my $moniker ($from->sources) {
+            my $source = $from->source($moniker);
+            my $new = $source->new($source);
+            # we use extra here as we want to leave the class_mappings as they are
+            # but overwrite the source_registrations entry with the new source
+            $self->register_extra_source($moniker => $new);
+        }
+
+        if ($from->storage) {
+            $self->storage($from->storage);
+            $self->storage->set_schema($self);
+        }
+    }
 }
 
 =head2 connection
@@ -198,10 +290,11 @@ as soon as the connection information is defined.
 =cut
 
 sub connection {
-    my $self = shift;
+    my $self  = shift;
+    my $class = ref $self || $self;
 
     if($_[-1] && ref $_[-1] eq 'HASH') {
-        for my $option (qw/ loader_class loader_options result_base_class schema_base_class/) {
+        for my $option (qw/loader_class loader_options/) {
             if(my $value = delete $_[-1]->{$option}) {
                 $self->$option($value);
             }
@@ -209,9 +302,66 @@ sub connection {
         pop @_ if !keys %{$_[-1]};
     }
 
-    $self = $self->next::method(@_);
+    # Make sure we inherit from schema_base_class and load schema_components
+    # before connecting.
+    require DBIx::Class::Schema::Loader::Base;
+    my $temp_loader = DBIx::Class::Schema::Loader::Base->new(
+        %{ $self->_loader_args },
+        schema => $self,
+        naming => 'current',
+        use_namespaces => 1,
+    );
+
+    my $modify_isa = 0;
+    my @components;
+
+    if ($temp_loader->schema_base_class || $temp_loader->schema_components) {
+        @components = @{ $temp_loader->schema_components }
+            if $temp_loader->schema_components;
+
+        push @components, ('+'.$temp_loader->schema_base_class)
+            if $temp_loader->schema_base_class;
+
+        my $class_isa = do {
+            no strict 'refs';
+            \@{"${class}::ISA"};
+        };
+
+        my @component_classes = map {
+            /^\+/ ? substr($_, 1, length($_) - 1) : "DBIx::Class::$_"
+        } @components;
+
+        $modify_isa++ if not array_eq([ @$class_isa[0..(@components-1)] ], \@component_classes)
+    }
+
+    if ($modify_isa) {
+        $class->load_components(@components);
+
+        # This hack is necessary because we changed @ISA of $self through
+        # ->load_components and we are now in a different place in the mro.
+        no warnings 'redefine';
+
+        local *connection = subname __PACKAGE__.'::connection' => sub {
+            my $self = shift;
+            $self->next::method(@_);
+        };
+
+        my @linear_isa = @{ mro::get_linear_isa($class) };
+
+        my $next_method;
+
+        foreach my $i (1..$#linear_isa) {
+            no strict 'refs';
+            $next_method = *{$linear_isa[$i].'::connection'}{CODE};
+            last if $next_method;
+        }
+
+        $self = $self->$next_method(@_);
+    }
+    else {
+        $self = $self->next::method(@_);
+    }
 
-    my $class = ref $self || $self;
     if(!$class->_loader_invoked) {
         $self->_invoke_loader
     }
@@ -338,6 +488,8 @@ memory at runtime without generating on-disk class files.
 For a complete list of supported loader_options, see
 L<DBIx::Class::Schema::Loader::Base>
 
+The last hashref in the C<\@connect_info> can specify the L</loader_class>.
+
 This function can be imported in the usual way, as illustrated in
 these Examples:
 
@@ -347,7 +499,9 @@ these Examples:
     make_schema_at(
         'New::Schema::Name',
         { debug => 1 },
-        [ 'dbi:Pg:dbname="foo"','postgres' ],
+        [ 'dbi:Pg:dbname="foo"','postgres','',
+          { loader_class => 'MyLoader' } # optionally
+        ],
     );
 
     # Inside a script, specifying a dump directory in which to write
@@ -356,9 +510,14 @@ these Examples:
     make_schema_at(
         'New::Schema::Name',
         { debug => 1, dump_directory => './lib' },
-        [ 'dbi:Pg:dbname="foo"','postgres' ],
+        [ 'dbi:Pg:dbname="foo"','postgres','',
+          { loader_class => 'MyLoader' } # optionally
+        ],
     );
 
+The last hashref in the C<\@connect_info> is checked for loader arguments such
+as C<loader_options> and C<loader_class>, see L</connection> for more details.
+
 =cut
 
 sub make_schema_at {
@@ -369,8 +528,16 @@ sub make_schema_at {
         @{$target . '::ISA'} = qw/DBIx::Class::Schema::Loader/;
     }
 
+    $target->_loader_invoked(0);
+
     $target->loader_options($opts);
-    $target->connection(@$connect_info);
+
+    my $temp_schema = $target->connect(@$connect_info);
+
+    $target->storage($temp_schema->storage);
+    $target->storage->set_schema($target);
+
+    return $target;
 }
 
 =head2 rescan
@@ -389,7 +556,7 @@ Returns a list of the new monikers added.
 
 =cut
 
-sub rescan { my $self = shift; $self->_loader->rescan($self) }
+sub rescan { my $self = shift; $self->loader->rescan($self) }
 
 =head2 naming
 
@@ -433,19 +600,7 @@ Can be imported into your dump script and called as a function as well:
 
 =head2 Multiple Database Schemas
 
-Currently the loader is limited to working within a single schema
-(using the underlying RDBMS's definition of "schema").  If you have a
-multi-schema database with inter-schema relationships (which is easy
-to do in PostgreSQL or DB2 for instance), you currently can only
-automatically load the tables of one schema, and relationships to
-tables in other schemas will be silently ignored.
-
-At some point in the future, an intelligent way around this might be
-devised, probably by allowing the C<db_schema> option to be an
-arrayref of schemas to load.
-
-In "normal" L<DBIx::Class::Schema> usage, manually-defined
-source classes and relationships have no problems crossing vendor schemas.
+See L<DBIx::Class::Schema::Loader::Base/db_schema>.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -462,25 +617,61 @@ blblack: Brandon Black <blblack@gmail.com>
 
 =head1 CONTRIBUTORS
 
-ilmarii: Dagfinn Ilmari Mannsåker <ilmari@ilmari.org>
+ilmari: Dagfinn Ilmari MannsE<aring>ker <ilmari@ilmari.org>
 
 arcanez: Justin Hunter <justin.d.hunter@gmail.com>
 
 ash: Ash Berlin <ash@cpan.org>
 
+btilly: Ben Tilly <btilly@gmail.com>
+
 Caelum: Rafael Kitover <rkitover@cpan.org>
 
 TSUNODA Kazuya <drk@drk7.jp>
 
-Robert Bohne <rbo@openserv.org>
+rbo: Robert Bohne <rbo@cpan.org>
 
-ribasushi: Peter Rabbitson <rabbit+dbic@rabbit.us>
+ribasushi: Peter Rabbitson <ribasushi@cpan.org>
 
 gugu: Andrey Kostenko <a.kostenko@rambler-co.ru>
 
 jhannah: Jay Hannah <jay@jays.net>
 
-rbuels: Robert Buels <rmb32@cornell.edu>
+jnap: John Napiorkowski <jjn1056@yahoo.com>
+
+rbuels: Robert Buels <rbuels@gmail.com>
+
+timbunce: Tim Bunce <timb@cpan.org>
+
+mst: Matt S. Trout <mst@shadowcatsystems.co.uk>
+
+mstratman: Mark A. Stratman <stratman@gmail.com>
+
+kane: Jos Boumans <kane@cpan.org>
+
+waawaamilk: Nigel McNie <nigel@mcnie.name>
+
+acmoore: Andrew Moore <amoore@cpan.org>
+
+bphillips: Brian Phillips <bphillips@cpan.org>
+
+schwern: Michael G. Schwern <mschwern@cpan.org>
+
+SineSwiper: Brendan Byrd <byrd.b@insightcom.com>
+
+hobbs: Andrew Rodland <arodland@cpan.org>
+
+domm: Thomas Klausner <domm@plix.at>
+
+spb: Stephen Bennett <spb@exherbo.org>
+
+Matias E. Fernandez <mfernandez@pisco.ch>
+
+alnewkirk: Al Newkirk <awncorp@cpan.org>
+
+angelixd: Paul C. Mantz <pcmantz@cpan.org>
+
+andrewalker: André Walker <andre@andrewalker.net>
 
 ... and lots of other folks. If we forgot you, please write the current
 maintainer or RT.
@@ -496,8 +687,10 @@ the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<DBIx::Class>, L<DBIx::Class::Manual::ExampleSchema>
+L<DBIx::Class>, L<DBIx::Class::Manual::Intro>, L<DBIx::Class::Tutorial>,
+L<DBIx::Class::Schema::Loader::Base>
 
 =cut
 
 1;
+# vim:et sts=4 sw=4 tw=0:
