@@ -3,21 +3,17 @@ package DBIx::Class::Schema::Loader::DBI::ODBC;
 use strict;
 use warnings;
 use base 'DBIx::Class::Schema::Loader::DBI';
-use Carp::Clan qw/^DBIx::Class/;
-use Class::C3;
+use mro 'c3';
 
-our $VERSION = '0.04999_14';
+our $VERSION = '0.07034_01';
 
 =head1 NAME
 
-DBIx::Class::Schema::Loader::DBI::ODBC - L<DBD::ODBC> proxy, currently only for
-Microsoft SQL Server
+DBIx::Class::Schema::Loader::DBI::ODBC - L<DBD::ODBC> proxy
 
 =head1 DESCRIPTION
 
-Reblesses into L<DBIx::Class::Schema::Loader::DBI::ODBC::Microsoft_SQL_Server>,
-which is a proxy for L<DBIx::Class::Schema::Loader::DBI::MSSQL> when using the
-L<DBD::ODBC> driver with Microsoft SQL Server.
+Reblesses into an C<::ODBC::> class when connecting via L<DBD::ODBC>.
 
 Code stolen from the L<DBIx::Class> ODBC storage.
 
@@ -27,9 +23,11 @@ See L<DBIx::Class::Schema::Loader::Base> for usage information.
 
 sub _rebless {
   my $self = shift;
-  my $dbh  = $self->schema->storage->dbh;
+
+  return if ref $self ne __PACKAGE__;
 
 # stolen from DBIC ODBC storage
+  my $dbh  = $self->schema->storage->dbh;
   my $dbtype = eval { $dbh->get_info(17) };
   unless ( $@ ) {
     # Translate the backend name into a perl identifier
@@ -42,10 +40,17 @@ sub _rebless {
   }
 }
 
+sub _tables_list {
+    my ($self, $opts) = @_;
+
+    return $self->next::method($opts, undef, undef);
+}
+
 =head1 SEE ALSO
 
 L<DBIx::Class::Schema::Loader::DBI::ODBC::Microsoft_SQL_Server>,
-L<DBIx::Class::Schema::Loader::DBI::MSSQL>,
+L<DBIx::Class::Schema::Loader::DBI::ODBC::SQL_Anywhere>,
+L<DBIx::Class::Schema::Loader::DBI::ODBC::Firebird>,
 L<DBIx::Class::Schema::Loader>, L<DBIx::Class::Schema::Loader::Base>,
 L<DBIx::Class::Schema::Loader::DBI>
 
